@@ -21,10 +21,14 @@
 #include "gd77_codeplug.hh"
 #include "gd77_filereader.hh"
 #include "opengd77_codeplug.hh"
+#include "openrtx_codeplug.hh"
+#include "anytone_filereader.hh"
 #include "d868uv_codeplug.hh"
 #include "d878uv_codeplug.hh"
 #include "d878uv2_codeplug.hh"
 #include "d578uv_codeplug.hh"
+#include "dmr6x2uv_codeplug.hh"
+
 
 int decodeCodeplug(QCommandLineParser &parser, QCoreApplication &app) {
   Q_UNUSED(app);
@@ -58,17 +62,17 @@ int decodeCodeplug(QCommandLineParser &parser, QCoreApplication &app) {
     if (parser.isSet("manufacturer")) {
       if (! MD390FileReader::read(filename, &codeplug, errorMessage)) {
         logError() << "Cannot decode manufacturer codeplug file '" << filename
-                   << "': " << errorMessage;
+                   << "':\n" << errorMessage;
         return -1;
       }
     } else if (! codeplug.read(filename, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "' : " << err.format();
+                 << "' :\n" << err.format();
       return -1;
     }
     if (! codeplug.decode(&config, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "': " << err.format();
+                 << "':\n" << err.format();
     }
   } else if (RadioInfo::UV390 == radio) {
     UV390Codeplug codeplug;
@@ -80,29 +84,29 @@ int decodeCodeplug(QCommandLineParser &parser, QCoreApplication &app) {
       }
     } else if (! codeplug.read(filename, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "' : " << err.format();
+                 << "' :\n" << err.format();
       return -1;
     }
     if (! codeplug.decode(&config, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "': " << err.format();
+                 << "':\n" << err.format();
     }
   } else if (RadioInfo::MD2017 == radio) {
     MD2017Codeplug codeplug;
     if (parser.isSet("manufacturer")) {
       if (! MD2017FileReader::read(filename, &codeplug, errorMessage)) {
         logError() << "Cannot decode manufacturer codeplug file '" << filename
-                   << "': " << errorMessage;
+                   << "':\n" << errorMessage;
         return -1;
       }
     } else if (! codeplug.read(filename, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "' : " << err.format();
+                 << "' :\n" << err.format();
       return -1;
     }
     if (! codeplug.decode(&config, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "': " << err.format();
+                 << "':\n" << err.format();
       return -1;
     }
   } else if (RadioInfo::DM1701 == radio) {
@@ -110,17 +114,17 @@ int decodeCodeplug(QCommandLineParser &parser, QCoreApplication &app) {
     if (parser.isSet("manufacturer")) {
       if (! DM1701FileReader::read(filename, &codeplug, errorMessage)) {
         logError() << "Cannot decode manufacturer codeplug file '" << filename
-                   << "': " << errorMessage;
+                   << "':\n" << errorMessage;
         return -1;
       }
     } else if (! codeplug.read(filename, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "' : " << err.format();
+                 << "' :\n" << err.format();
       return -1;
     }
     if (! codeplug.decode(&config, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "': " << err.format();
+                 << "':\n" << err.format();
       return -1;
     }
   } else if (RadioInfo::RD5R == radio) {
@@ -128,17 +132,17 @@ int decodeCodeplug(QCommandLineParser &parser, QCoreApplication &app) {
     if (parser.isSet("manufacturer")) {
       if (! RD5RFileReader::read(filename, &codeplug, errorMessage)) {
         logError() << "Cannot decode manufacturer codeplug file '" << filename
-                   << "': " << errorMessage;
+                   << "':\n" << errorMessage;
         return -1;
       }
     } else if (! codeplug.read(filename, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "': " << err.format();
+                 << "':\n" << err.format();
       return -1;
     }
     if (! codeplug.decode(&config, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "': " << err.format();
+                 << "':\n" << err.format();
       return -1;
     }
   } else if (RadioInfo::GD77 == radio) {
@@ -146,17 +150,17 @@ int decodeCodeplug(QCommandLineParser &parser, QCoreApplication &app) {
     if (parser.isSet("manufacturer")) {
       if (! GD77FileReader::read(filename, &codeplug, errorMessage)) {
         logError() << "Cannot decode manufacturer codeplug file '" << filename
-                   << "': " << errorMessage;
+                   << "':\n" << errorMessage;
         return -1;
       }
     } else if (! codeplug.read(filename, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "': " << err.format();
+                 << "':\n" << err.format();
       return -1;
     }
     if (! codeplug.decode(&config, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "': " << err.format();
+                 << "':\n" << err.format();
       return -1;
     }
   } else if (RadioInfo::OpenGD77 == radio) {
@@ -168,12 +172,29 @@ int decodeCodeplug(QCommandLineParser &parser, QCoreApplication &app) {
     OpenGD77Codeplug codeplug;
     if (! codeplug.read(filename, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "': " << err.format();
+                 << "':\n" << err.format();
       return -1;
     }
     if (! codeplug.decode(&config, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "': " << err.format();
+                 << "':\n" << err.format();
+      return -1;
+    }
+  } else if (RadioInfo::OpenRTX == radio) {
+    if (parser.isSet("manufacturer")) {
+      logError() << "Decoding of manufacturer codeplug is not implemented for radio '" <<
+                    RadioInfo::byID(radio).name() << "'.";
+      return -1;
+    }
+    OpenRTXCodeplug codeplug;
+    if (! codeplug.read(filename, err)) {
+      logError() << "Cannot decode binary codeplug file '" << filename
+                 << "':\n" << err.format();
+      return -1;
+    }
+    if (! codeplug.decode(&config, err)) {
+      logError() << "Cannot decode binary codeplug file '" << filename
+                 << "':\n" << err.format();
       return -1;
     }
   } else if (RadioInfo::D868UVE == radio) {
@@ -185,12 +206,12 @@ int decodeCodeplug(QCommandLineParser &parser, QCoreApplication &app) {
     }
     if (! codeplug.read(filename, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "':" << err.format();
+                 << "':\n" << err.format();
       return -1;
     }
     if (! codeplug.decode(&config, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "':" << err.format();
+                 << "':\n" << err.format();
       return -1;
     }
   } else if (RadioInfo::D878UV == radio) {
@@ -202,12 +223,12 @@ int decodeCodeplug(QCommandLineParser &parser, QCoreApplication &app) {
     D878UVCodeplug codeplug;
     if (! codeplug.read(filename, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "':" << err.format();
+                 << "':\n" << err.format();
       return -1;
     }
     if (! codeplug.decode(&config, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "':" << err.format();
+                 << "':\n" << err.format();
       return -1;
     }
   } else if (RadioInfo::D878UVII == radio) {
@@ -219,12 +240,12 @@ int decodeCodeplug(QCommandLineParser &parser, QCoreApplication &app) {
     D878UV2Codeplug codeplug;
     if (! codeplug.read(filename, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename <<
-                    "':" << err.format();
+                    "':\n" << err.format();
       return -1;
     }
     if (! codeplug.decode(&config, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "':" << err.format();
+                 << "':\n" << err.format();
       return -1;
     }
   } else if (RadioInfo::D578UV == radio) {
@@ -236,12 +257,29 @@ int decodeCodeplug(QCommandLineParser &parser, QCoreApplication &app) {
     D578UVCodeplug codeplug;
     if (! codeplug.read(filename, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "':" << err.format();
+                 << "':\n" << err.format();
       return -1;
     }
     if (! codeplug.decode(&config, err)) {
       logError() << "Cannot decode binary codeplug file '" << filename
-                 << "':" << err.format();
+                 << "':\n" << err.format();
+      return -1;
+    }
+  } else if (RadioInfo::DMR6X2UV == radio) {
+    if (parser.isSet("manufacturer")) {
+      logError() << "Decoding of manufacturer codeplug is not implemented for radio '" <<
+                    RadioInfo::byID(radio).name() << "'.";
+      return -1;
+    }
+    DMR6X2UVCodeplug codeplug;
+    if (! codeplug.read(filename, err)) {
+      logError() << "Cannot decode binary codeplug file '" << filename
+                 << "':\n" << err.format();
+      return -1;
+    }
+    if (! codeplug.decode(&config, err)) {
+      logError() << "Cannot decode binary codeplug file '" << filename
+                 << "':\n" << err.format();
       return -1;
     }
   } else {
@@ -259,12 +297,12 @@ int decodeCodeplug(QCommandLineParser &parser, QCoreApplication &app) {
       QFile outfile(info.filePath());
       if (! outfile.open(QIODevice::WriteOnly)) {
         logError() << "Cannot write CSV codeplug file '" << outfile.fileName()
-                   << "': " << outfile.errorString();
+                   << "':\n" << outfile.errorString();
         return -1;
       }
       QTextStream stream(&outfile);
       if (! config.toYAML(stream, err)) {
-        logError() << "Cannot serialize codeplug to YAML:"
+        logError() << "Cannot serialize codeplug to YAML:\n"
                    << err.format(" ");
       }
       outfile.close();
@@ -280,7 +318,7 @@ int decodeCodeplug(QCommandLineParser &parser, QCoreApplication &app) {
       return -1;
     } else if (parser.isSet("yaml")) {
       if (! config.toYAML(stream, err)) {
-        logError() << "Cannot serialize codeplug into YAML:"
+        logError() << "Cannot serialize codeplug into YAML:\n"
                    << err.format(" ");
         return -1;
       }
