@@ -229,6 +229,81 @@ ConfigItem::copy(const ConfigItem &other) {
   return true;
 }
 
+int
+ConfigItem::compare(const ConfigItem &other) const {
+  // Check if other has the same type
+  if (strcmp(other.metaObject()->className(), metaObject()->className()))
+    return strcmp(metaObject()->className(), other.metaObject()->className());
+
+  // Compare by properties
+  const QMetaObject *meta = metaObject();
+  for (int p=QObject::staticMetaObject.propertyCount(); p<meta->propertyCount(); p++) {
+    // This property
+    QMetaProperty prop = meta->property(p);
+    // the same property over at other
+    QMetaProperty oprop = other.metaObject()->property(p);
+
+    // Should never happen
+    if (! prop.isValid())
+      continue;
+
+    // Handle comparison of basic types
+    if ((prop.isEnumType()) || (QVariant::Bool == prop.type()) || (QVariant::Int == prop.type()) || (QVariant::UInt == prop.type())) {
+      int a=prop.read(this).toInt(), b=oprop.read(&other).toInt();
+      if (a<b) return -1;
+      if (a>b) return 1;
+      continue;
+    }
+
+    if (QVariant::Double == prop.type()) {
+      double a=prop.read(this).toDouble(), b=oprop.read(&other).toDouble();
+      if (a<b) return -1;
+      if (a>b) return 1;
+      continue;
+    }
+
+    if (QVariant::String == prop.type()) {
+      int cmp = QString::compare(prop.read(this).toString(), oprop.read(&other).toString());
+      if (cmp) return cmp;
+      continue;
+    }
+
+    if (ConfigObjectReference *ref = prop.read(this).value<ConfigObjectReference *>()) {
+      int cmp = ref->compare(*oprop.read(&other).value<ConfigObjectReference*>());
+      if (cmp) return cmp;
+      continue;
+    }
+
+    if (ConfigObjectList *lst = prop.read(this).value<ConfigObjectList *>()) {
+      int cmp = lst->compare(*oprop.read(&other).value<ConfigObjectList*>());
+      if (cmp) return cmp;
+      continue;
+    }
+
+    if (ConfigObjectRefList *lst = prop.read(this).value<ConfigObjectRefList *>()) {
+      int cmp = lst->compare(*oprop.read(&other).value<ConfigObjectRefList*>());
+      if (cmp) return cmp;
+      continue;
+    }
+
+    if (propIsInstance<ConfigItem>(prop)) {
+      // If the owned item is writeable -> clone if set in other
+      if (prop.read(&other).isNull() && !oprop.read(&other).isNull())
+        return -1;
+      if (!prop.read(&other).isNull() && oprop.read(&other).isNull())
+        return 1;
+      if (prop.read(&other).isNull() && oprop.read(&other).isNull())
+        continue;
+      int cmp = prop.read(&other).value<ConfigItem*>()->compare(*oprop.read(&other).value<ConfigItem*>());
+      if (cmp) return cmp;
+      continue;
+    }
+  }
+
+  return 0;
+}
+
+
 bool
 ConfigItem::label(ConfigObject::Context &context, const ErrorStack &err) {
   // Label properties owning config objects, that is of type ConfigObject or ConfigObjectList
@@ -265,7 +340,7 @@ void
 ConfigItem::clear() {
   emit beginClear();
 
-  // Delete or clear all object owned by properites, that is ConfigObjectList and ConfigObject
+  // Delete or clear all object owned by properties, that is ConfigObjectList and ConfigObject
   const QMetaObject *meta = metaObject();
   for (int p=QObject::staticMetaObject.propertyCount(); p<meta->propertyCount(); p++) {
     QMetaProperty prop = meta->property(p);
@@ -519,8 +594,8 @@ ConfigItem::parse(const YAML::Node &node, ConfigItem::Context &ctx, const ErrorS
       // check type
       if (! node[prop.name()].IsMap()) {
         errMsg(err) << node[prop.name()].Mark().line << ":" << node[prop.name()].Mark().column
-                    << ": Cannot parse " << prop.name() << " of " << meta->className()
-                    << ": Expected instance of '"
+                    << ": Cannot parse '" << prop.name() << "' of '" << meta->className()
+                    << "': Expected instance of '"
                     << QMetaType::metaObjectForType(prop.userType())->className() << "'.";
         return false;
       }
@@ -696,7 +771,7 @@ ConfigItem::link(const YAML::Node &node, const ConfigItem::Context &ctx, const E
           if (0 > lst->add(ctx.getTag(prop.enclosingMetaObject()->className(), prop.name(), tag))) {
             errMsg(err) << it->Mark().line << ":" << it->Mark().column
                         << ": Cannot link " << prop.name() << " of " << meta->className()
-                        << ": Cannot add referece for tag '" << tag << "'.";
+                        << ": Cannot add reference for tag '" << tag << "'.";
             return false;
           }
           continue;
@@ -860,14 +935,14 @@ ConfigItem::longDescription(const QMetaProperty &prop) const {
 /* ********************************************************************************************* *
  * Implementation of ConfigObject
  * ********************************************************************************************* */
-ConfigObject::ConfigObject(const QString &idBase, QObject *parent)
-  : ConfigItem(parent), _idBase(idBase), _name()
+ConfigObject::ConfigObject(QObject *parent)
+  : ConfigItem(parent), _name()
 {
   // pass...
 }
 
-ConfigObject::ConfigObject(const QString &name, const QString &idBase, QObject *parent)
-  : ConfigItem(parent), _idBase(idBase), _name(name)
+ConfigObject::ConfigObject(const QString &name, QObject *parent)
+  : ConfigItem(parent), _name(name)
 {
   // pass...
 }
@@ -885,16 +960,18 @@ ConfigObject::setName(const QString &name) {
   emit modified(this);
 }
 
+QString
+ConfigObject::idPrefix() const {
+  return findIdPrefix(this->metaObject());
+}
+
 bool
 ConfigObject::label(ConfigObject::Context &context, const ErrorStack &err) {
-  // With empty ID base, skip labeling.
-  if (_idBase.isEmpty())
-    return true;
-
   unsigned n=1;
-  QString id=QString("%1%2").arg(_idBase).arg(n);
+  QString prefix = this->idPrefix();
+  QString id=QString("%1%2").arg(prefix).arg(n);
   while (context.contains(id)) {
-    id=QString("%1%2").arg(_idBase).arg(++n);
+    id=QString("%1%2").arg(prefix).arg(++n);
   }
 
   if (! context.add(id, this)) {
@@ -935,6 +1012,18 @@ ConfigObject::populate(YAML::Node &node, const Context &context, const ErrorStac
   return ConfigItem::populate(node, context, err);
 }
 
+QString
+ConfigObject::findIdPrefix(const QMetaObject *meta) {
+  for (int i=meta->classInfoOffset(); i<meta->classInfoCount(); i++) {
+    if (0 == strcmp("IdPrefix", meta->classInfo(i).name())) {
+      return meta->classInfo(i).value();
+    }
+  }
+  if (meta->superClass())
+    return findIdPrefix(meta->superClass());
+  return "";
+}
+
 
 /* ********************************************************************************************* *
  * Implementation of ConfigExtension
@@ -1014,7 +1103,7 @@ int AbstractConfigObjectList::add(ConfigObject *obj, int row) {
   // Ignore nullptr
   if (nullptr == obj)
     return -1;
-  // If alread in list -> ignore
+  // If already in list -> ignore
   if (0 <= indexOf(obj))
     return -1;
   if (-1 == row)
@@ -1264,6 +1353,19 @@ ConfigObjectList::copy(const AbstractConfigObjectList &other) {
   return true;
 }
 
+int
+ConfigObjectList::compare(const ConfigObjectList &other) const {
+  if (count() < other.count())
+    return -1;
+  if (count() > other.count())
+    return 1;
+  for (int i=0; i<count(); i++) {
+    int cmp = this->get(i)->compare(*other.get(i));
+    if (cmp) return cmp;
+  }
+
+  return 0;
+}
 
 /* ********************************************************************************************* *
  * Implementation of ConfigObjectRefList
@@ -1300,3 +1402,14 @@ ConfigObjectRefList::serialize(const ConfigItem::Context &context, const ErrorSt
   return list;
 }
 
+int
+ConfigObjectRefList::compare(const ConfigObjectRefList &other) const {
+  if (count() < other.count()) return -1;
+  if (count() > other.count()) return 1;
+  for (int i=0; i<count(); i++) {
+    int cmp = get(i)->compare(*other.get(i));
+    if (cmp) return cmp;
+  }
+  return 0;
+}
+
