@@ -6,7 +6,7 @@
 #include "zone.hh"
 #include "scanlist.hh"
 #include "gpssystem.hh"
-#include "roaming.hh"
+#include "roamingzone.hh"
 
 
 D578UVLimits::D578UVLimits(const std::initializer_list<std::pair<double, double> > &rxFreqRanges,
