@@ -44,9 +44,14 @@ D868UVCallsignDB::EntryElement::setNumber(unsigned num) {
   setBCD8_be(0x0001, num);
 }
 
+void
+D868UVCallsignDB::EntryElement::setFriendFlag(bool set) {
+  setBit(0x0005, 3, set);
+}
+
 void
 D868UVCallsignDB::EntryElement::setRingTone(RingTone tone) {
-  setUInt8(0x0005, (unsigned)tone);
+  setUInt2(0x0005, 0, (unsigned)tone);
 }
 
 void
