@@ -177,20 +177,34 @@ D878UVCodeplug::ChannelElement::setFrequencyCorrection(int corr) {
 
 Channel *
 D878UVCodeplug::ChannelElement::toChannelObj(Context &ctx) const {
-  Channel *ch = AnytoneCodeplug::ChannelElement::toChannelObj(ctx);
+  Channel *ch = D868UVCodeplug::ChannelElement::toChannelObj(ctx);
+
   if (nullptr == ch)
     return nullptr;
-  // Nothing else to do
+
+  // Get extensions
+  AnytoneChannelExtension *ext = nullptr;
+  if (DMRChannel *dch = ch->as<DMRChannel>()) {
+    ext = dch->anytoneChannelExtension();
+  } else if (FMChannel *fch = ch->as<FMChannel>()){
+    ext = fch->anytoneChannelExtension();
+  }
+
+  // If extension is present, update
+  if (nullptr != ext) {
+    ext->setFrequencyCorrection(frequenyCorrection());
+  }
+
   return ch;
 }
 
 bool
 D878UVCodeplug::ChannelElement::linkChannelObj(Channel *c, Context &ctx) const {
-  if (! AnytoneCodeplug::ChannelElement::linkChannelObj(c, ctx))
+  if (! D868UVCodeplug::ChannelElement::linkChannelObj(c, ctx))
     return false;
 
-  if (c->is<DigitalChannel>()) {
-    DigitalChannel *dc = c->as<DigitalChannel>();
+  if (c->is<DMRChannel>()) {
+    DMRChannel *dc = c->as<DMRChannel>();
     // Link to GPS system
     if (txDigitalAPRS() && ctx.has<GPSSystem>(digitalAPRSSystemIndex()))
       dc->setAPRSObj(ctx.get<GPSSystem>(digitalAPRSSystemIndex()));
@@ -201,8 +215,8 @@ D878UVCodeplug::ChannelElement::linkChannelObj(Channel *c, Context &ctx) const {
     // If roaming is not disabled -> link to default roaming zone
     if (roamingEnabled())
       dc->setRoamingZone(DefaultRoamingZone::get());
-  } else if (c->is<AnalogChannel>()) {
-    AnalogChannel *ac = c->as<AnalogChannel>();
+  } else if (c->is<FMChannel>()) {
+    FMChannel *ac = c->as<FMChannel>();
     // Link APRS system if one is defined
     //  There can only be one active APRS system, hence the index is fixed to one.
     if (txAnalogAPRS() && ctx.has<APRSSystem>(0))
@@ -214,10 +228,10 @@ D878UVCodeplug::ChannelElement::linkChannelObj(Channel *c, Context &ctx) const {
 
 bool
 D878UVCodeplug::ChannelElement::fromChannelObj(const Channel *c, Context &ctx) {
-  if (! AnytoneCodeplug::ChannelElement::fromChannelObj(c, ctx))
+  if (! D868UVCodeplug::ChannelElement::fromChannelObj(c, ctx))
     return false;
 
-  if (const DigitalChannel *dc = c->as<DigitalChannel>()) {
+  if (const DMRChannel *dc = c->as<DMRChannel>()) {
     // Set GPS system index
     enableRXAPRS(false);
     if (dc->aprsObj() && dc->aprsObj()->is<GPSSystem>()) {
@@ -231,13 +245,24 @@ D878UVCodeplug::ChannelElement::fromChannelObj(const Channel *c, Context &ctx) {
     // Enable roaming
     if (dc->roaming())
       enableRoaming(true);
-  } else if (const AnalogChannel *ac = c->as<AnalogChannel>()) {
+    // Apply extension settings, if present
+    if (AnytoneDMRChannelExtension *ext = dc->anytoneChannelExtension()) {
+      setFrequencyCorrection(ext->frequencyCorrection());
+      /// Handles bug in AnyTone firmware.
+      /// @todo Remove once fixed by AnyTone.
+      enableRXAPRS(! ext->sms());
+    }
+  } else if (const FMChannel *ac = c->as<FMChannel>()) {
     // Set APRS system
     enableRXAPRS(false);
     if (nullptr != ac->aprsSystem()) {
       enableTXAnalogAPRS(true);
       enableRXAPRS(true);
     }
+    // Apply extension settings
+    if (AnytoneFMChannelExtension *ext = ac->anytoneChannelExtension()) {
+      setFrequencyCorrection(ext->frequencyCorrection());
+    }
   }
 
   return true;
@@ -266,82 +291,92 @@ D878UVCodeplug::RoamingChannelElement::clear() {
 
 unsigned
 D878UVCodeplug::RoamingChannelElement::rxFrequency() const {
-  return getBCD8_be(0x0000)*10;
+  return getBCD8_be(Offsets::RXFrequency)*10;
 }
 void
 D878UVCodeplug::RoamingChannelElement::setRXFrequency(unsigned hz) {
-  setBCD8_be(0x0000, hz/10);
+  setBCD8_be(Offsets::RXFrequency, hz/10);
 }
 unsigned
 D878UVCodeplug::RoamingChannelElement::txFrequency() const {
-  return getBCD8_be(0x0004)*10;
+  return getBCD8_be(Offsets::TXFrequency)*10;
 }
 void
 D878UVCodeplug::RoamingChannelElement::setTXFrequency(unsigned hz) {
-  setBCD8_be(0x0004, hz/10);
+  setBCD8_be(Offsets::TXFrequency, hz/10);
+}
+
+bool
+D878UVCodeplug::RoamingChannelElement::hasColorCode() const {
+  return ColorCodeValue::Disabled == getUInt8(Offsets::ColorCode);
 }
 unsigned
 D878UVCodeplug::RoamingChannelElement::colorCode() const {
-  return getUInt8(0x0008);
+  return std::min(15u, (unsigned)getUInt8(Offsets::ColorCode));
 }
 void
 D878UVCodeplug::RoamingChannelElement::setColorCode(unsigned cc) {
-  setUInt8(0x0008, cc);
-}
-
-DigitalChannel::TimeSlot
-D878UVCodeplug::RoamingChannelElement::timeSlot() const {
-  switch (getUInt8(0x0009)) {
-  case 0x00: return DigitalChannel::TimeSlot::TS1;
-  case 0x01: return DigitalChannel::TimeSlot::TS2;
-  }
-  return DigitalChannel::TimeSlot::TS1;
+  setUInt8(Offsets::ColorCode, cc);
 }
 void
-D878UVCodeplug::RoamingChannelElement::setTimeSlot(DigitalChannel::TimeSlot ts) {
+D878UVCodeplug::RoamingChannelElement::disableColorCode() {
+  setUInt8(Offsets::ColorCode, ColorCodeValue::Disabled);
+}
+
+DMRChannel::TimeSlot
+D878UVCodeplug::RoamingChannelElement::timeSlot() const {
+  switch (getUInt8(Offsets::TimeSlot)) {
+  case TimeSlotValue::TS1: return DMRChannel::TimeSlot::TS1;
+  case TimeSlotValue::TS2: return DMRChannel::TimeSlot::TS2;
+  }
+  return DMRChannel::TimeSlot::TS1;
+}
+void
+D878UVCodeplug::RoamingChannelElement::setTimeSlot(DMRChannel::TimeSlot ts) {
   switch (ts) {
-  case DigitalChannel::TimeSlot::TS1: setUInt8(0x0009, 0x00); break;
-  case DigitalChannel::TimeSlot::TS2: setUInt8(0x0009, 0x01); break;
+  case DMRChannel::TimeSlot::TS1: setUInt8(Offsets::TimeSlot, TimeSlotValue::TS1); break;
+  case DMRChannel::TimeSlot::TS2: setUInt8(Offsets::TimeSlot, TimeSlotValue::TS2); break;
   }
 }
 
 QString
 D878UVCodeplug::RoamingChannelElement::name() const {
-  return readASCII(0x000a, 16, 0x00);
+  return readASCII(Offsets::Name, Offsets::NameLength, 0x00);
 }
 void
 D878UVCodeplug::RoamingChannelElement::setName(const QString &name) {
-  writeASCII(0x000a, name, 16, 0x00);
+  writeASCII(Offsets::Name, name, Offsets::NameLength, 0x00);
 }
 
 bool
-D878UVCodeplug::RoamingChannelElement::fromChannel(const DigitalChannel *ch) {
+D878UVCodeplug::RoamingChannelElement::fromChannel(const RoamingChannel* ch) {
   setName(ch->name());
   setRXFrequency(ch->rxFrequency()*1e6);
   setTXFrequency(ch->txFrequency()*1e6);
-  setColorCode(ch->colorCode());
+  if (ch->colorCodeOverridden())
+    setColorCode(ch->colorCode());
+  else
+    disableColorCode();
   setTimeSlot(ch->timeSlot());
   return true;
 }
 
-DigitalChannel *
+RoamingChannel *
 D878UVCodeplug::RoamingChannelElement::toChannel(Context &ctx) {
-  // Find matching channel for RX, TX frequency, TS and CC
-  DigitalChannel *digi = ctx.config()->channelList()->findDigitalChannel(
-        rxFrequency()/1e6, txFrequency()/1e6, timeSlot(), colorCode());
-  if (nullptr == digi) {
-    // If no matching channel can be found -> create one
-    digi = new DigitalChannel();
-    digi->setName(name());
-    digi->setRXFrequency(rxFrequency()/1e6);
-    digi->setTXFrequency(txFrequency()/1e6);
-    digi->setColorCode(colorCode());
-    digi->setTimeSlot(timeSlot());
-    logDebug() << "No matching roaming channel found: Create channel '"
-               << digi->name() << "' as roaming channel.";
-    ctx.config()->channelList()->add(digi);
-  }
-  return digi;
+  RoamingChannel *roam = new RoamingChannel();
+  roam->setName(name());
+  roam->setRXFrequency(rxFrequency()/1e6);
+  roam->setTXFrequency(txFrequency()/1e6);
+  if (hasColorCode())
+    roam->setColorCode(colorCode());
+  else
+    roam->overrideColorCode(false);
+  roam->overrideTimeSlot(true);
+  roam->setTimeSlot(timeSlot());
+
+  ctx.config()->roamingChannels()->add(roam);
+
+  return roam;
 }
 
 
@@ -394,12 +429,12 @@ D878UVCodeplug::RoamingZoneElement::setName(const QString &name) {
 
 bool
 D878UVCodeplug::RoamingZoneElement::fromRoamingZone(
-    RoamingZone *zone, const QHash<DigitalChannel *, unsigned> &map)
+    RoamingZone *zone, Context &ctx)
 {
   clear();
   setName(zone->name());
   for (int i=0; i<std::min(NUM_CH_PER_ROAMINGZONE, zone->count()); i++) {
-    setMember(i, map.value(zone->channel(i), 0xff));
+    setMember(i, ctx.index(zone->channel(i)));
   }
   return true;
 }
@@ -410,20 +445,11 @@ D878UVCodeplug::RoamingZoneElement::toRoamingZone() const {
 }
 
 bool
-D878UVCodeplug::RoamingZoneElement::linkRoamingZone(
-    RoamingZone *zone, const QHash<unsigned, DigitalChannel*> &map)
+D878UVCodeplug::RoamingZoneElement::linkRoamingZone(RoamingZone *zone, Context &ctx)
 {
   for (uint8_t i=0; (i<NUM_CH_PER_ROAMINGZONE)&&hasMember(i); i++) {
-    DigitalChannel *digi = nullptr;
-    if (map.contains(member(i))) {
-      // If matching index is known -> resolve directly
-      digi = map.value(member(i));
-    } else {
-      logError() << "Cannot link roaming zone '" << zone->name()
-                 << "', unknown roaming channel index " << member(i);
-      return false;
-    }
-    zone->addChannel(digi);
+    if (ctx.has<RoamingChannel>(i))
+      zone->addChannel(ctx.get<RoamingChannel>(i));
   }
   return true;
 }
@@ -1598,17 +1624,17 @@ D878UVCodeplug::AnalogAPRSSettingsElement::toAPRSSystem() {
 bool
 D878UVCodeplug::AnalogAPRSSettingsElement::linkAPRSSystem(APRSSystem *sys, Context &ctx) {
   // First, try to find a matching analog channel in list
-  AnalogChannel *ch = ctx.config()->channelList()->findAnalogChannelByTxFreq(frequency()/1e6);
+  FMChannel *ch = ctx.config()->channelList()->findFMChannelByTxFreq(frequency()/1e6);
   if (! ch) {
     // If no channel is found, create one with the settings from APRS channel:
-    ch = new AnalogChannel();
+    ch = new FMChannel();
     ch->setName("APRS Channel");
     ch->setRXFrequency(frequency()/1e6);
     ch->setTXFrequency(frequency()/1e6);
     ch->setPower(power());
     ch->setTXTone(txTone());
-    ch->setBandwidth(AnalogChannel::Bandwidth::Wide);
-    logInfo() << "No matching APRS chanel found for TX frequency " << frequency()/1e6
+    ch->setBandwidth(FMChannel::Bandwidth::Wide);
+    logInfo() << "No matching APRS channel found for TX frequency " << frequency()/1e6
               << "MHz, create one as 'APRS Channel'";
     ctx.config()->channelList()->add(ch);
   }
@@ -1815,21 +1841,21 @@ D878UVCodeplug::DMRAPRSSystemsElement::setDestination(unsigned n, unsigned idx)
   setBCD8_be(0x0010 + 4*n, idx);
 }
 
-DigitalContact::Type
+DMRContact::Type
 D878UVCodeplug::DMRAPRSSystemsElement::callType(unsigned n) const {
   switch(getUInt8(0x0030 + n)) {
-  case 0: return DigitalContact::PrivateCall;
-  case 1: return DigitalContact::GroupCall;
-  case 2: return DigitalContact::AllCall;
+  case 0: return DMRContact::PrivateCall;
+  case 1: return DMRContact::GroupCall;
+  case 2: return DMRContact::AllCall;
   }
-  return DigitalContact::PrivateCall;
+  return DMRContact::PrivateCall;
 }
 void
-D878UVCodeplug::DMRAPRSSystemsElement::setCallType(unsigned n, DigitalContact::Type type) {
+D878UVCodeplug::DMRAPRSSystemsElement::setCallType(unsigned n, DMRContact::Type type) {
   switch(type) {
-  case DigitalContact::PrivateCall: setUInt8(0x0030+n, 0x00); break;
-  case DigitalContact::GroupCall: setUInt8(0x0030+n, 0x01); break;
-  case DigitalContact::AllCall: setUInt8(0x0030+n, 0x02); break;
+  case DMRContact::PrivateCall: setUInt8(0x0030+n, 0x00); break;
+  case DMRContact::GroupCall: setUInt8(0x0030+n, 0x01); break;
+  case DMRContact::AllCall: setUInt8(0x0030+n, 0x02); break;
   }
 }
 
@@ -1837,19 +1863,19 @@ bool
 D878UVCodeplug::DMRAPRSSystemsElement::timeSlotOverride(unsigned n) {
   return 0x00 != getUInt8(0x0039 + n);
 }
-DigitalChannel::TimeSlot
+DMRChannel::TimeSlot
 D878UVCodeplug::DMRAPRSSystemsElement::timeSlot(unsigned n) const {
   switch (getUInt8(0x0039 + n)) {
-  case 1: return DigitalChannel::TimeSlot::TS1;
-  case 2: return DigitalChannel::TimeSlot::TS2;
+  case 1: return DMRChannel::TimeSlot::TS1;
+  case 2: return DMRChannel::TimeSlot::TS2;
   }
-  return DigitalChannel::TimeSlot::TS1;
+  return DMRChannel::TimeSlot::TS1;
 }
 void
-D878UVCodeplug::DMRAPRSSystemsElement::setTimeSlot(unsigned n, DigitalChannel::TimeSlot ts) {
+D878UVCodeplug::DMRAPRSSystemsElement::setTimeSlot(unsigned n, DMRChannel::TimeSlot ts) {
   switch (ts) {
-  case DigitalChannel::TimeSlot::TS1: setUInt8(0x0039+n, 0x01); break;
-  case DigitalChannel::TimeSlot::TS2: setUInt8(0x0039+n, 0x02); break;
+  case DMRChannel::TimeSlot::TS1: setUInt8(0x0039+n, 0x01); break;
+  case DMRChannel::TimeSlot::TS2: setUInt8(0x0039+n, 0x02); break;
   }
 }
 void
@@ -1917,14 +1943,14 @@ D878UVCodeplug::DMRAPRSSystemsElement::linkGPSSystem(int idx, GPSSystem *sys, Co
   // if a revert channel is defined -> link to it
   if (channelIsSelected(idx))
     sys->setRevertChannel(nullptr);
-  else if (ctx.has<Channel>(channelIndex(idx)) && ctx.get<Channel>(channelIndex(idx))->is<DigitalChannel>())
-    sys->setRevertChannel(ctx.get<Channel>(channelIndex(idx))->as<DigitalChannel>());
+  else if (ctx.has<Channel>(channelIndex(idx)) && ctx.get<Channel>(channelIndex(idx))->is<DMRChannel>())
+    sys->setRevertChannel(ctx.get<Channel>(channelIndex(idx))->as<DMRChannel>());
 
   // Search for a matching contact in contacts
-  DigitalContact *cont = ctx.config()->contacts()->findDigitalContact(destination(idx));
+  DMRContact *cont = ctx.config()->contacts()->findDigitalContact(destination(idx));
   // If no matching contact is found, create one
   if (nullptr == cont) {
-    cont = new DigitalContact(callType(idx), tr("GPS #%1 Contact").arg(idx+1),
+    cont = new DMRContact(callType(idx), tr("GPS #%1 Contact").arg(idx+1),
                               destination(idx), false);
     ctx.config()->contacts()->add(cont);
   }
@@ -2115,21 +2141,29 @@ D878UVCodeplug::RadioInfoElement::maintainerNote() const {
 /* ******************************************************************************************** *
  * Implementation of D878UVCodeplug
  * ******************************************************************************************** */
-D878UVCodeplug::D878UVCodeplug(QObject *parent)
-  : D868UVCodeplug(parent)
+D878UVCodeplug::D878UVCodeplug(const QString &label, QObject *parent)
+  : D868UVCodeplug(label, parent)
 {
-  // Rename image
-  image(0).setName("Anytone AT-D878UV Codeplug");
+  // pass...
+}
+
+D878UVCodeplug::D878UVCodeplug(QObject *parent)
+  : D868UVCodeplug("Anytone AT-D878UV Codeplug", parent)
+{
+  // pass...
+}
+
+bool
+D878UVCodeplug::allocateBitmaps() {
+  if (! D868UVCodeplug::allocateBitmaps())
+    return false;
 
   // Roaming channel bitmaps
   image(0).addElement(ADDR_ROAMING_CHANNEL_BITMAP, ROAMING_CHANNEL_BITMAP_SIZE);
   // Roaming zone bitmaps
   image(0).addElement(ADDR_ROAMING_ZONE_BITMAP, ROAMING_ZONE_BITMAP_SIZE);
-}
 
-void
-D878UVCodeplug::clear() {
-  D868UVCodeplug::clear();
+  return true;
 }
 
 void
@@ -2171,16 +2205,14 @@ D878UVCodeplug::setBitmaps(Config *config)
   // Mark roaming zones
   uint8_t *roaming_zone_bitmap = data(ADDR_ROAMING_ZONE_BITMAP);
   memset(roaming_zone_bitmap, 0x00, ROAMING_ZONE_BITMAP_SIZE);
-  for (int i=0; i<config->roaming()->count(); i++)
+  for (int i=0; i<config->roamingZones()->count(); i++)
     roaming_zone_bitmap[i/8] |= (1<<(i%8));
 
   // Mark roaming channels
   uint8_t *roaming_ch_bitmap = data(ADDR_ROAMING_CHANNEL_BITMAP);
   memset(roaming_ch_bitmap, 0x00, ROAMING_CHANNEL_BITMAP_SIZE);
   // Get all (unique) channels used in roaming
-  QSet<DigitalChannel*> roaming_channels;
-  config->roaming()->uniqueChannels(roaming_channels);
-  for (int i=0; i<std::min(NUM_ROAMING_CHANNEL,roaming_channels.count()); i++)
+  for (int i=0; i<std::min(NUM_ROAMING_CHANNEL,config->roamingChannels()->count()); i++)
     roaming_ch_bitmap[i/8] |= (1<<(i%8));
 }
 
@@ -2405,7 +2437,7 @@ D878UVCodeplug::createGPSSystems(Context &ctx, const ErrorStack &err) {
 
   // replaces D868UVCodeplug::createGPSSystems
 
-  // Before creating any GPS/APRS systems, get global auto TX intervall
+  // Before creating any GPS/APRS systems, get global auto TX interval
   AnalogAPRSSettingsElement aprs(data(ADDR_APRS_SETTING));
   unsigned pos_intervall = aprs.autoTXInterval();
 
@@ -2495,29 +2527,28 @@ D878UVCodeplug::encodeRoaming(const Flags &flags, Context &ctx, const ErrorStack
   Q_UNUSED(flags); Q_UNUSED(err)
 
   // Encode roaming channels
-  QHash<DigitalChannel *, unsigned> roaming_ch_map;
-  {
-    // Get set of unique roaming channels
-    QSet<DigitalChannel*> roaming_channels;
-    ctx.config()->roaming()->uniqueChannels(roaming_channels);
-    // Encode channels and store in index<->channel map
-    int i=0; QSet<DigitalChannel*>::iterator ch=roaming_channels.begin();
-    for(; ch != roaming_channels.end(); ch++, i++) {
-      roaming_ch_map[*ch] = i;
-      uint32_t addr = ADDR_ROAMING_CHANNEL_0+i*ROAMING_CHANNEL_OFFSET;
-      RoamingChannelElement rch(data(addr));
-      rch.fromChannel(*ch);
+  for (uint8_t i=0; i<std::min(NUM_ROAMING_CHANNEL, ctx.config()->roamingChannels()->count()); i++) {
+    // Encode roaming channel
+    uint32_t addr = ADDR_ROAMING_CHANNEL_0 + i*ROAMING_CHANNEL_OFFSET;
+    RoamingChannelElement rch_elm(data(addr));
+    RoamingChannel *rch = ctx.config()->roamingChannels()->get(i)->as<RoamingChannel>();
+    rch_elm.clear();
+    rch_elm.fromChannel(rch);
+    if (! ctx.add(rch, i)) {
+      errMsg(err) << "Cannot add index " << i << " for roaming channel '"
+                  << rch->name() << "' to codeplug context.";
+      return false;
     }
   }
 
   // Encode roaming zones
-  for (int i=0; i<ctx.config()->roaming()->count(); i++){
+  for (int i=0; i<ctx.config()->roamingZones()->count(); i++){
     uint32_t addr = ADDR_ROAMING_ZONE_0+i*ROAMING_ZONE_OFFSET;
     RoamingZoneElement zone(data(addr));
-    logDebug() << "Encode roaming zone " << ctx.config()->roaming()->zone(i)->name()
+    logDebug() << "Encode roaming zone " << ctx.config()->roamingZones()->zone(i)->name()
                << " (" << (i+1) << ") at " << QString::number(addr, 16)
-               << " with " << ctx.config()->roaming()->zone(i)->count() << " elements.";
-    zone.fromRoamingZone(ctx.config()->roaming()->zone(i), roaming_ch_map);
+               << " with " << ctx.config()->roamingZones()->zone(i)->count() << " elements.";
+    zone.fromRoamingZone(ctx.config()->roamingZones()->zone(i), ctx);
   }
 
   return true;
@@ -2527,7 +2558,6 @@ bool
 D878UVCodeplug::createRoaming(Context &ctx, const ErrorStack &err) {
   Q_UNUSED(err)
 
-  QHash<unsigned, DigitalChannel*> map;
   // Create or find roaming channels
   uint8_t *roaming_channel_bitmap = data(ADDR_ROAMING_CHANNEL_BITMAP);
   for (int i=0; i<NUM_ROAMING_CHANNEL; i++) {
@@ -2536,8 +2566,8 @@ D878UVCodeplug::createRoaming(Context &ctx, const ErrorStack &err) {
       continue;
     uint32_t addr = ADDR_ROAMING_CHANNEL_0 + i*ROAMING_CHANNEL_OFFSET;
     RoamingChannelElement ch(data(addr));
-    if (DigitalChannel *digi = ch.toChannel(ctx))
-      map.insert(i, digi);
+    RoamingChannel *digi = ch.toChannel(ctx);
+    ctx.add(digi, i);
   }
 
   // Create and link roaming zones
@@ -2549,8 +2579,8 @@ D878UVCodeplug::createRoaming(Context &ctx, const ErrorStack &err) {
     uint32_t addr = ADDR_ROAMING_ZONE_0 + i*ROAMING_ZONE_OFFSET;
     RoamingZoneElement z(data(addr));
     RoamingZone *zone = z.toRoamingZone();
-    ctx.config()->roaming()->add(zone); ctx.add(zone, i);
-    z.linkRoamingZone(zone, map);
+    ctx.config()->roamingZones()->add(zone); ctx.add(zone, i);
+    z.linkRoamingZone(zone, ctx);
   }
 
   return true;
