@@ -6,7 +6,7 @@
 #include "scanlist.hh"
 #include "zone.hh"
 #include "gpssystem.hh"
-#include "roaming.hh"
+#include "roamingzone.hh"
 
 
 DM1701Limits::DM1701Limits(QObject *parent)
@@ -14,7 +14,7 @@ DM1701Limits::DM1701Limits(QObject *parent)
 {
   // Define limits for call-sign DB
   _hasCallSignDB          = true;
-  _callSignDBImplemented  = false;
+  _callSignDBImplemented  = true;
   _numCallSignDBEntries   = 122197;
 
   add("settings", new RadioLimitItem {
