@@ -230,11 +230,11 @@ MD390Codeplug::createChannels(Config *config, Context &ctx, const ErrorStack &er
   for (int i=0; i<NUM_CHANNELS; i++) {
     ChannelElement chan(data(ADDR_CHANNELS+i*CHANNEL_SIZE));
     if (! chan.isValid())
-      break;
+      continue;
     if (Channel *obj = chan.toChannelObj()) {
       config->channelList()->add(obj); ctx.add(obj, i+1);
     } else {
-      errMsg(err) << "Invlaid channel at index " << i << ".";
+      errMsg(err) << "Invalid channel at index " << i << ".";
       return false;
     }
   }
@@ -246,7 +246,7 @@ MD390Codeplug::linkChannels(Context &ctx, const ErrorStack &err) {
   for (int i=0; i<NUM_CHANNELS; i++) {
     ChannelElement chan(data(ADDR_CHANNELS+i*CHANNEL_SIZE));
     if (! chan.isValid())
-      break;
+      continue;
     if (! chan.linkChannelObj(ctx.get<Channel>(i+1), ctx)) {
       errMsg(err) << "Cannot link channel at index " << i << ".";
       return false;
@@ -281,11 +281,11 @@ MD390Codeplug::createContacts(Config *config, Context &ctx, const ErrorStack &er
   for (int i=0; i<NUM_CONTACTS; i++) {
     ContactElement cont(data(ADDR_CONTACTS+i*CONTACT_SIZE));
     if (! cont.isValid())
-      break;
-    if (DigitalContact *obj = cont.toContactObj()) {
+      continue;
+    if (DMRContact *obj = cont.toContactObj()) {
       config->contacts()->add(obj); ctx.add(obj, i+1);
     } else {
-      errMsg(err) << "Invlaid contact at index " << i << ".";
+      errMsg(err) << "Invalid contact at index " << i << ".";
       return false;
     }
   }
@@ -345,7 +345,7 @@ MD390Codeplug::createZones(Config *config, Context &ctx, const ErrorStack &err)
   for (int i=0; i<NUM_ZONES; i++) {
     ZoneElement zone(data(ADDR_ZONES+i*ZONE_SIZE));
     if (! zone.isValid())
-      break;
+      continue;
     bool is_ext = (nullptr != last_zone) && (zone.name().endsWith(" B")) &&
         (zone.name().startsWith(last_zone->name()));
     Zone *obj = last_zone;
@@ -367,8 +367,7 @@ MD390Codeplug::linkZones(Context &ctx, const ErrorStack &err) {
   for (int i=0, z=0; i<NUM_ZONES; i++, z++) {
     ZoneElement zone(data(ADDR_ZONES+i*ZONE_SIZE));
     if (! zone.isValid())
-      break;
-
+      continue;
     if (ctx.has<Zone>(i+1)) {
       Zone *obj = last_zone = ctx.get<Zone>(i+1);
       for (int i=0; ((i<16) && zone.memberIndex(i)); i++) {
@@ -419,11 +418,11 @@ MD390Codeplug::createGroupLists(Config *config, Context &ctx, const ErrorStack &
   for (int i=0; i<NUM_GROUPLISTS; i++) {
     GroupListElement glist(data(ADDR_GROUPLISTS+i*GROUPLIST_SIZE));
     if (! glist.isValid())
-      break;
+      continue;
     if (RXGroupList *obj = glist.toGroupListObj(ctx)) {
       config->rxGroupLists()->add(obj); ctx.add(obj, i+1);
     } else {
-      errMsg(err) << "Invlaid group list at index " << i << ".";
+      errMsg(err) << "Invalid group list at index " << i << ".";
       return false;
     }
   }
@@ -435,7 +434,7 @@ MD390Codeplug::linkGroupLists(Context &ctx, const ErrorStack &err) {
   for (int i=0; i<NUM_GROUPLISTS; i++) {
     GroupListElement glist(data(ADDR_GROUPLISTS+i*GROUPLIST_SIZE));
     if (! glist.isValid())
-      break;
+      continue;
     if (! glist.linkGroupListObj(ctx.get<RXGroupList>(i+1), ctx)) {
       errMsg(err) << "Cannot link group list at index " << i << ".";
       return false;
@@ -470,11 +469,11 @@ MD390Codeplug::createScanLists(Config *config, Context &ctx, const ErrorStack &e
   for (int i=0; i<NUM_SCANLISTS; i++) {
     ScanListElement scan(data(ADDR_SCANLISTS + i*SCANLIST_SIZE));
     if (! scan.isValid())
-      break;
+      continue;
     if (ScanList *obj = scan.toScanListObj(ctx)) {
       config->scanlists()->add(obj); ctx.add(obj, i+1);
     } else {
-      errMsg(err) << "Invlaid scanlist at index " << i << ".";
+      errMsg(err) << "Invalid scanlist at index " << i << ".";
       return false;
     }
   }
@@ -486,8 +485,7 @@ MD390Codeplug::linkScanLists(Context &ctx, const ErrorStack &err) {
   for (int i=0; i<NUM_SCANLISTS; i++) {
     ScanListElement scan(data(ADDR_SCANLISTS + i*SCANLIST_SIZE));
     if (! scan.isValid())
-      break;
-
+      continue;
     if (! scan.linkScanListObj(ctx.get<ScanList>(i+1), ctx)) {
       errMsg(err) << "Cannot link scan list at index " << i << ".";
       return false;
@@ -525,11 +523,11 @@ MD390Codeplug::createPositioningSystems(Config *config, Context &ctx, const Erro
   for (int i=0; i<NUM_GPSSYSTEMS; i++) {
     GPSSystemElement gps(data(ADDR_GPSSYSTEMS+i*GPSSYSTEM_SIZE));
     if (! gps.isValid())
-      break;
+      continue;
     if (GPSSystem *obj = gps.toGPSSystemObj()) {
       config->posSystems()->add(obj); ctx.add(obj, i+1);
     } else {
-      errMsg(err) << "Invlaid GPS system at index " << i << ".";
+      errMsg(err) << "Invalid GPS system at index " << i << ".";
       return false;
     }
   }
@@ -542,7 +540,7 @@ MD390Codeplug::linkPositioningSystems(Context &ctx, const ErrorStack &err) {
   for (int i=0; i<NUM_GPSSYSTEMS; i++) {
     GPSSystemElement gps(data(ADDR_GPSSYSTEMS+i*GPSSYSTEM_SIZE));
     if (! gps.isValid())
-      break;
+      continue;
     if (! gps.linkGPSSystemObj(ctx.get<GPSSystem>(i+1), ctx)) {
       errMsg(err) << "Cannot link GPS system at index " << i << ".";
       return false;
@@ -570,6 +568,37 @@ MD390Codeplug::decodeButtonSetttings(Config *config, const ErrorStack &err) {
   return ButtonSettingsElement(data(ADDR_BUTTONSETTINGS)).updateConfig(config);
 }
 
+
+void
+MD390Codeplug::clearPrivacyKeys() {
+  EncryptionElement(data(ADDR_PRIVACY_KEYS)).clear();
+}
+
+bool
+MD390Codeplug::encodePrivacyKeys(Config *config, const Flags &flags, Context &ctx, const ErrorStack &err) {
+  Q_UNUSED(flags); Q_UNUSED(err);
+  // First, reset keys
+  clearPrivacyKeys();
+  // Get keys
+  EncryptionElement keys(data(ADDR_PRIVACY_KEYS));
+  // Encode keys
+  return keys.fromCommercialExt(config->commercialExtension(), ctx);
+}
+
+bool
+MD390Codeplug::decodePrivacyKeys(Config *config, Context &ctx, const ErrorStack &err) {
+  Q_UNUSED(config)
+  // Get keys
+  EncryptionElement keys(data(ADDR_PRIVACY_KEYS));
+  // Decode element
+  if (! keys.updateCommercialExt(ctx)) {
+    errMsg(err) << "Cannot create encryption extension.";
+    return false;
+  }
+  return true;
+}
+
+
 void
 MD390Codeplug::clearMenuSettings() {
   MenuSettingsElement(data(ADDR_MENUSETTINGS)).clear();
@@ -580,12 +609,6 @@ MD390Codeplug::clearTextMessages() {
   memset(data(ADDR_TEXTMESSAGES), 0, NUM_TEXTMESSAGES*TEXTMESSAGE_SIZE);
 }
 
-void
-MD390Codeplug::clearPrivacyKeys() {
-  EncryptionElement(data(ADDR_PRIVACY_KEYS)).clear();
-
-}
-
 void
 MD390Codeplug::clearEmergencySystems() {
   EmergencySettingsElement(data(ADDR_EMERGENCY_SETTINGS)).clear();
