@@ -10,6 +10,9 @@
 #define ADDR_BUTTONS              0x000108
 #define ADDR_MESSAGES             0x000128
 
+#define ADDR_ENCRYPTION           0x001370
+#define ENCRYPTION_SIZE               0x88
+
 #define NUM_CONTACTS                   256
 #define ADDR_CONTACTS             0x001788
 #define CONTACT_SIZE              0x000018
@@ -79,8 +82,8 @@ RD5RCodeplug::ChannelElement::fromChannelObj(const Channel *c, Context &ctx) {
   if (! RadioddityCodeplug::ChannelElement::fromChannelObj(c, ctx))
     return false;
 
-  if (c->is<AnalogChannel>()) {
-    const AnalogChannel *ac = c->as<AnalogChannel>();
+  if (c->is<FMChannel>()) {
+    const FMChannel *ac = c->as<FMChannel>();
     if (ac->defaultSquelch())
       setSquelch(ctx.config()->settings()->squelch());
     else if (ac->squelchDisabled())
@@ -101,8 +104,8 @@ RD5RCodeplug::ChannelElement::toChannelObj(Context &ctx) const {
   if (nullptr == ch)
     return nullptr;
 
-  if (ch->is<AnalogChannel>()) {
-    AnalogChannel *ac = ch->as<AnalogChannel>();
+  if (ch->is<FMChannel>()) {
+    FMChannel *ac = ch->as<FMChannel>();
     ac->setSquelch(squelch());
   }
 
@@ -163,6 +166,39 @@ RD5RCodeplug::TimestampElement::set(const QDateTime &ts) {
 }
 
 
+/* ******************************************************************************************** *
+ * Implementation of RD5RCodeplug::EncryptionElement
+ * ******************************************************************************************** */
+RD5RCodeplug::EncryptionElement::EncryptionElement(uint8_t *ptr)
+  : RadioddityCodeplug::EncryptionElement(ptr)
+{
+  // pass...
+}
+
+bool
+RD5RCodeplug::EncryptionElement::isBasicKeySet(unsigned n) const {
+  if (n>0)
+    return false;
+  return RadioddityCodeplug::EncryptionElement::isBasicKeySet(n);
+}
+
+QByteArray
+RD5RCodeplug::EncryptionElement::basicKey(unsigned n) const {
+  if (n>0)
+    return QByteArray();
+  return QByteArray("\x53\x47\x4c\x39");
+}
+
+void
+RD5RCodeplug::EncryptionElement::setBasicKey(unsigned n, const QByteArray &key) {
+  if ((0 != n) || (key != "\x53\x47\x4c\x39")){
+    logError() << "The RD5R only supports a single fixed DMR basic key '53474c39'.";
+    return;
+  }
+  RD5RCodeplug::EncryptionElement::setBasicKey(n, key);
+}
+
+
 /* ******************************************************************************************** *
  * Implementation of RD5RCodeplug
  * ******************************************************************************************** */
@@ -271,7 +307,7 @@ RD5RCodeplug::createContacts(Config *config, Context &ctx, const ErrorStack &err
     if (!el.isValid())
       continue;
 
-    DigitalContact *cont = el.toContactObj(ctx);
+    DMRContact *cont = el.toContactObj(ctx);
     ctx.add(cont, i+1); config->contacts()->add(cont);
   }
   return true;
@@ -601,7 +637,12 @@ RD5RCodeplug::encodeGroupLists(Config *config, const Flags &flags, Context &ctx,
       continue;
     GroupListElement el(bank.get(i));
     el.fromRXGroupListObj(config->rxGroupLists()->list(i), ctx);
-    bank.setContactCount(i, config->rxGroupLists()->list(i)->count());
+    // Only group calls are encoded
+    int count = 0;
+    for (int j=0; j<config->rxGroupLists()->list(i)->count(); j++)
+      if (DMRContact::GroupCall == config->rxGroupLists()->list(i)->contact(j)->type())
+        count++;
+    bank.setContactCount(i, count);
   }
   return true;
 }
@@ -639,3 +680,33 @@ RD5RCodeplug::linkGroupLists(Config *config, Context &ctx, const ErrorStack &err
   }
   return true;
 }
+
+
+void
+RD5RCodeplug::clearEncryption() {
+  EncryptionElement enc(data(ADDR_ENCRYPTION));
+  enc.clear();
+}
+
+bool
+RD5RCodeplug::encodeEncryption(Config *config, const Flags &flags, Context &ctx, const ErrorStack &err) {
+  Q_UNUSED(flags); Q_UNUSED(err);
+  clearEncryption();
+  EncryptionElement enc(data(ADDR_ENCRYPTION));
+  return enc.fromCommercialExt(config->commercialExtension(), ctx);
+}
+
+bool
+RD5RCodeplug::createEncryption(Config *config, Context &ctx, const ErrorStack &err) {
+  Q_UNUSED(config); Q_UNUSED(err);
+  EncryptionElement enc(data(ADDR_ENCRYPTION));
+  if (EncryptionElement::PrivacyType::None == enc.privacyType())
+    return true;
+  return enc.updateCommercialExt(ctx);
+}
+
+bool
+RD5RCodeplug::linkEncryption(Config *config, Context &ctx, const ErrorStack &err) {
+  Q_UNUSED(config); Q_UNUSED(ctx); Q_UNUSED(err);
+  return true;
+}
