@@ -1,5 +1,4 @@
 #include "tyt_codeplug.hh"
-#include "codeplugcontext.hh"
 #include "config.hh"
 #include "utils.hh"
 #include "channel.hh"
@@ -7,9 +6,10 @@
 #include "config.h"
 #include "logger.hh"
 #include "tyt_extensions.hh"
+#include "encryptionextension.hh"
 #include <QTimeZone>
 #include <QtEndian>
-
+#include <QChar>
 
 #define CHANNEL_SIZE      0x000040
 #define SETTINGS_SIZE     0x000090
@@ -38,7 +38,7 @@ TyTCodeplug::ChannelElement::~ChannelElement() {
 
 bool
 TyTCodeplug::ChannelElement::isValid() const {
-  return Element::isValid() && (0x0000 != getUInt16_be(0x20)) && (0xffff != getUInt16_be(0x20));
+  return Element::isValid() && QChar::isPrint(getUInt16_le(0x20));
 }
 
 void
@@ -46,13 +46,13 @@ TyTCodeplug::ChannelElement::clear() {
   Element::clear();
 
   setMode(MODE_ANALOG);
-  setBandwidth(AnalogChannel::Bandwidth::Narrow);
+  setBandwidth(FMChannel::Bandwidth::Narrow);
   enableAutoScan(0);
   setBit(0, 1); setBit(0,2);
   enableLoneWorker(false);
   enableTalkaround(false);
   enableRXOnly(false);
-  setTimeSlot(DigitalChannel::TimeSlot::TS1);
+  setTimeSlot(DMRChannel::TimeSlot::TS1);
   setColorCode(1);
   setPrivacyIndex(0);
   setPrivacyType(PRIV_NONE);
@@ -102,15 +102,15 @@ TyTCodeplug::ChannelElement::setMode(Mode mode) {
   setUInt2(0,0, uint8_t(mode));
 }
 
-AnalogChannel::Bandwidth
+FMChannel::Bandwidth
 TyTCodeplug::ChannelElement::bandwidth() const {
   if (0 == getUInt2(0, 2))
-    return AnalogChannel::Bandwidth::Narrow;
-  return AnalogChannel::Bandwidth::Wide;
+    return FMChannel::Bandwidth::Narrow;
+  return FMChannel::Bandwidth::Wide;
 }
 void
-TyTCodeplug::ChannelElement::setBandwidth(AnalogChannel::Bandwidth bw) {
-  if (AnalogChannel::Bandwidth::Narrow == bw)
+TyTCodeplug::ChannelElement::setBandwidth(FMChannel::Bandwidth bw) {
+  if (FMChannel::Bandwidth::Narrow == bw)
     setUInt2(0, 2, BW_12_5_KHZ);
   else
     setUInt2(0, 2, BW_25_KHZ);
@@ -152,15 +152,15 @@ TyTCodeplug::ChannelElement::enableRXOnly(bool enable) {
   setBit(1, 1, enable);
 }
 
-DigitalChannel::TimeSlot
+DMRChannel::TimeSlot
 TyTCodeplug::ChannelElement::timeSlot() const {
   if (2 == getUInt2(1,2))
-    return DigitalChannel::TimeSlot::TS2;
-  return DigitalChannel::TimeSlot::TS1;
+    return DMRChannel::TimeSlot::TS2;
+  return DMRChannel::TimeSlot::TS1;
 }
 void
-TyTCodeplug::ChannelElement::setTimeSlot(DigitalChannel::TimeSlot ts) {
-  if (DigitalChannel::TimeSlot::TS1 == ts)
+TyTCodeplug::ChannelElement::setTimeSlot(DMRChannel::TimeSlot ts) {
+  if (DMRChannel::TimeSlot::TS1 == ts)
     setUInt2(1,2,1);
   else
     setUInt2(1,2,2);
@@ -253,8 +253,6 @@ TyTCodeplug::ChannelElement::vox() const {
 }
 void
 TyTCodeplug::ChannelElement::enableVOX(bool enable) {
-  if (enable)
-    logDebug() << "Enable VOX!";
   setBit(4,4, enable);
 }
 
@@ -420,23 +418,25 @@ TyTCodeplug::ChannelElement::setName(const QString &name) {
 }
 
 Channel *
-TyTCodeplug::ChannelElement::toChannelObj() const {
-  if (! isValid())
+TyTCodeplug::ChannelElement::toChannelObj(const ErrorStack &err) const {
+  if (! isValid()) {
+    errMsg(err) << "Cannot decode invalid channel.";
     return nullptr;
+  }
 
   Channel *ch = nullptr;
   TyTChannelExtension *ex = new TyTChannelExtension();
 
   // decode power setting
   if (MODE_ANALOG == mode()) {
-    AnalogChannel::Admit admit_crit;
+    FMChannel::Admit admit_crit;
     switch(admitCriterion()) {
-      case ADMIT_ALWAYS: admit_crit = AnalogChannel::Admit::Always; break;
-      case ADMIT_TONE: admit_crit = AnalogChannel::Admit::Tone; break;
-      case ADMIT_CH_FREE: admit_crit = AnalogChannel::Admit::Free; break;
-      default: admit_crit = AnalogChannel::Admit::Free; break;
+      case ADMIT_ALWAYS: admit_crit = FMChannel::Admit::Always; break;
+      case ADMIT_TONE: admit_crit = FMChannel::Admit::Tone; break;
+      case ADMIT_CH_FREE: admit_crit = FMChannel::Admit::Free; break;
+      default: admit_crit = FMChannel::Admit::Free; break;
     }
-    AnalogChannel *ach = new AnalogChannel();
+    FMChannel *ach = new FMChannel();
     ach->setAdmit(admit_crit);
     ach->setSquelchDefault();
     ach->setRXTone(rxSignaling());
@@ -446,14 +446,14 @@ TyTCodeplug::ChannelElement::toChannelObj() const {
     ex->enableDisplayPTTId(displayPTTId());
     ch = ach;
   } else if (MODE_DIGITAL == mode()) {
-    DigitalChannel::Admit admit_crit;
+    DMRChannel::Admit admit_crit;
     switch(admitCriterion()) {
-      case ADMIT_ALWAYS: admit_crit = DigitalChannel::Admit::Always; break;
-      case ADMIT_CH_FREE: admit_crit = DigitalChannel::Admit::Free; break;
-      case ADMIT_COLOR: admit_crit = DigitalChannel::Admit::ColorCode; break;
-      default: admit_crit = DigitalChannel::Admit::Free; break;
+      case ADMIT_ALWAYS: admit_crit = DMRChannel::Admit::Always; break;
+      case ADMIT_CH_FREE: admit_crit = DMRChannel::Admit::Free; break;
+      case ADMIT_COLOR: admit_crit = DMRChannel::Admit::ColorCode; break;
+      default: admit_crit = DMRChannel::Admit::Free; break;
     }
-    DigitalChannel *dch = new DigitalChannel();
+    DMRChannel *dch = new DMRChannel();
     dch->setAdmit(admit_crit);
     dch->setColorCode(colorCode());
     dch->setTimeSlot(timeSlot());
@@ -461,7 +461,17 @@ TyTCodeplug::ChannelElement::toChannelObj() const {
     ex->enablePrivateCallConfirmed(privateCallConfirm());
     ex->enableDataCallConfirmed(dataCallConfirm());
     ex->enableEmergencyAlarmConfirmed(emergencyAlarmACK());
+
+    // If encryption is enabled, Add commercial extension to channel if needed
+    // the key will be linked later
+    if ((PRIV_NONE != privacyType()) && (nullptr == dch->commercialExtension()))
+      dch->setCommercialExtension(new CommercialChannelExtension());
+
+    // done
     ch = dch;
+  } else {
+    errMsg(err) << "Cannot decode channel. Channel type " << mode() << " unknown!";
+    return nullptr;
   }
 
   // Common settings
@@ -489,21 +499,24 @@ TyTCodeplug::ChannelElement::toChannelObj() const {
 }
 
 bool
-TyTCodeplug::ChannelElement::linkChannelObj(Channel *c, Context &ctx) const
+TyTCodeplug::ChannelElement::linkChannelObj(Channel *c, Context &ctx, const ErrorStack &err) const
 {
-  if (! isValid())
+  if (! isValid()) {
+    errMsg(err) << "Cannot link an invalid channel.";
     return false;
+  }
 
   if (scanListIndex() && ctx.has<ScanList>(scanListIndex())) {
-    c->setScanListObj(ctx.get<ScanList>(scanListIndex()));
+    c->setScanList(ctx.get<ScanList>(scanListIndex()));
   }
 
   if (MODE_ANALOG == mode()) {
+    // Nothing further to link for analog channels.
     return true;
-  } else if ((MODE_DIGITAL == mode()) && (c->is<DigitalChannel>())){
-    DigitalChannel *dc = c->as<DigitalChannel>();
-    if (contactIndex() && ctx.has<DigitalContact>(contactIndex())) {
-      dc->setTXContactObj(ctx.get<DigitalContact>(contactIndex()));
+  } else if ((MODE_DIGITAL == mode()) && (c->is<DMRChannel>())){
+    DMRChannel *dc = c->as<DMRChannel>();
+    if (contactIndex() && ctx.has<DMRContact>(contactIndex())) {
+      dc->setTXContactObj(ctx.get<DMRContact>(contactIndex()));
     }
     if (groupListIndex() && ctx.has<RXGroupList>(groupListIndex())) {
       dc->setGroupListObj(ctx.get<RXGroupList>(groupListIndex()));
@@ -511,9 +524,39 @@ TyTCodeplug::ChannelElement::linkChannelObj(Channel *c, Context &ctx) const
     if (positioningSystemIndex() && ctx.has<GPSSystem>(positioningSystemIndex())) {
       dc->setAPRSObj(ctx.get<GPSSystem>(positioningSystemIndex()));
     }
+
+    // Link encryption key if defined
+    if (PRIV_NONE != privacyType()) {
+      if (nullptr == dc->commercialExtension()) {
+        errMsg(err) << "Cannot link encryption key: No commercial extension set.";
+        return false;
+      }
+      if (PRIV_BASIC == privacyType()) {
+        if (! ctx.has<DMREncryptionKey>(privacyIndex())) {
+          errMsg(err) << "Cannot link encryption key: No basic key with index " << privacyIndex()
+                      << " defined.";
+          return false;
+        }
+        dc->commercialExtension()->setEncryptionKey(ctx.get<DMREncryptionKey>(privacyIndex()));
+      } else if (PRIV_ENHANCED == privacyType()) {
+        if (! ctx.has<AESEncryptionKey>(privacyIndex())) {
+          errMsg(err) << "Cannot link encryption key: No AES (enhances) key with index "
+                      << privacyIndex() << " defined.";
+          return false;
+        }
+        dc->commercialExtension()->setEncryptionKey(ctx.get<AESEncryptionKey>(privacyIndex()));
+      } else {
+        errMsg(err) << "Unknown encryption key type " << privacyType() << ".";
+        return false;
+      }
+    }
+
     return true;
   }
 
+  errMsg(err) << "Cannot link channel '" << c->name()
+              << "' invalid channel type " << mode() << ".";
+
   return false;
 }
 
@@ -527,8 +570,8 @@ TyTCodeplug::ChannelElement::fromChannelObj(const Channel *chan, Context &ctx) {
     setTXTimeOut(ctx.config()->settings()->tot());
   else
     setTXTimeOut(chan->timeout());
-  if (chan->scanListObj())
-    setScanListIndex(ctx.index(chan->scanListObj()));
+  if (chan->scanList())
+    setScanListIndex(ctx.index(chan->scanList()));
   else
     setScanListIndex(0);
   // Enable vox
@@ -537,13 +580,13 @@ TyTCodeplug::ChannelElement::fromChannelObj(const Channel *chan, Context &ctx) {
   enableVOX(defaultVOXEnabled || channelVOXEnabled);
   // power setting must be set by specialized element
 
-  if (chan->is<const DigitalChannel>()) {
-    const DigitalChannel *dchan = chan->as<const DigitalChannel>();
+  if (chan->is<const DMRChannel>()) {
+    const DMRChannel *dchan = chan->as<const DMRChannel>();
     setMode(MODE_DIGITAL);
     switch (dchan->admit()) {
-    case DigitalChannel::Admit::Always: setAdmitCriterion(ADMIT_ALWAYS); break;
-    case DigitalChannel::Admit::Free: setAdmitCriterion(ADMIT_CH_FREE); break;
-    case DigitalChannel::Admit::ColorCode: setAdmitCriterion(ADMIT_COLOR); break;
+    case DMRChannel::Admit::Always: setAdmitCriterion(ADMIT_ALWAYS); break;
+    case DMRChannel::Admit::Free: setAdmitCriterion(ADMIT_CH_FREE); break;
+    case DMRChannel::Admit::ColorCode: setAdmitCriterion(ADMIT_COLOR); break;
     }
     setColorCode(dchan->colorCode());
     setTimeSlot(dchan->timeSlot());
@@ -553,7 +596,7 @@ TyTCodeplug::ChannelElement::fromChannelObj(const Channel *chan, Context &ctx) {
       setGroupListIndex(0);
     if (dchan->txContactObj())
       setContactIndex(ctx.index(dchan->txContactObj()));
-    setBandwidth(AnalogChannel::Bandwidth::Narrow);
+    setBandwidth(FMChannel::Bandwidth::Narrow);
     setRXSignaling(Signaling::SIGNALING_NONE);
     setTXSignaling(Signaling::SIGNALING_NONE);
     if (dchan->aprsObj() && dchan->aprsObj()->is<GPSSystem>()) {
@@ -565,15 +608,34 @@ TyTCodeplug::ChannelElement::fromChannelObj(const Channel *chan, Context &ctx) {
       enableDataCallConfirm(chan->tytChannelExtension()->dataCallConfirmed());
       enableEmergencyAlarmACK(chan->tytChannelExtension()->emergencyAlarmConfirmed());
     }
-  } else if (chan->is<AnalogChannel>()) {
-    const AnalogChannel *achan = chan->as<const AnalogChannel>();
+    // Link encryption key if set
+    if (dchan->commercialExtension() && dchan->commercialExtension()->encryptionKey()) {
+      // Check for index
+      if (0 > ctx.index(dchan->commercialExtension()->encryptionKey())) {
+        logError() << "Cannot encode encryption key '"
+                   << dchan->commercialExtension()->encryptionKey()->name()
+                   << "': Not indexed.";
+      } else if (dchan->commercialExtension()->encryptionKey()->is<DMREncryptionKey>()) {
+        setPrivacyType(PRIV_BASIC);
+        setPrivacyIndex(ctx.index(dchan->commercialExtension()->encryptionKey()));
+      } else if (dchan->commercialExtension()->encryptionKey()->is<AESEncryptionKey>()) {
+        setPrivacyType(PRIV_ENHANCED);
+        setPrivacyIndex(ctx.index(dchan->commercialExtension()->encryptionKey()));
+      } else {
+        logInfo() << "Ignore unknown encryption key type "
+                  << dchan->commercialExtension()->encryptionKey()->metaObject()->className()
+                  << " for DMR channel.";
+      }
+    }
+  } else if (chan->is<FMChannel>()) {
+    const FMChannel *achan = chan->as<const FMChannel>();
     setMode(MODE_ANALOG);
     setBandwidth(achan->bandwidth());
     // Squelch must be set by specialized element
     switch (achan->admit()) {
-    case AnalogChannel::Admit::Always: setAdmitCriterion(ADMIT_ALWAYS); break;
-    case AnalogChannel::Admit::Free: setAdmitCriterion(ADMIT_CH_FREE); break;
-    case AnalogChannel::Admit::Tone: setAdmitCriterion(ADMIT_TONE); break;
+    case FMChannel::Admit::Always: setAdmitCriterion(ADMIT_ALWAYS); break;
+    case FMChannel::Admit::Free: setAdmitCriterion(ADMIT_CH_FREE); break;
+    case FMChannel::Admit::Tone: setAdmitCriterion(ADMIT_TONE); break;
     }
     setRXSignaling(achan->rxTone());
     setTXSignaling(achan->txTone());
@@ -649,22 +711,22 @@ TyTCodeplug::ContactElement::enableRingTone(bool enable) {
   setBit(3, 5, enable);
 }
 
-DigitalContact::Type TyTCodeplug::ContactElement::callType() const {
+DMRContact::Type TyTCodeplug::ContactElement::callType() const {
   switch(getUInt2(3,0)) {
-  case 1: return DigitalContact::GroupCall;
-  case 2: return DigitalContact::PrivateCall;
-  case 3: return DigitalContact::AllCall;
+  case 1: return DMRContact::GroupCall;
+  case 2: return DMRContact::PrivateCall;
+  case 3: return DMRContact::AllCall;
   default:
     break;
   }
-  return DigitalContact::PrivateCall;
+  return DMRContact::PrivateCall;
 }
 void
-TyTCodeplug::ContactElement::setCallType(DigitalContact::Type type) {
+TyTCodeplug::ContactElement::setCallType(DMRContact::Type type) {
   switch (type) {
-  case DigitalContact::GroupCall:   setUInt2(3,0, 1); break;
-  case DigitalContact::PrivateCall: setUInt2(3,0, 2); break;
-  case DigitalContact::AllCall:     setUInt2(3,0, 3); break;
+  case DMRContact::GroupCall:   setUInt2(3,0, 1); break;
+  case DMRContact::PrivateCall: setUInt2(3,0, 2); break;
+  case DMRContact::AllCall:     setUInt2(3,0, 3); break;
   }
 }
 
@@ -677,13 +739,13 @@ TyTCodeplug::ContactElement::setName(const QString &nm) {
   writeUnicode(4, nm, 16, 0x0000);
 }
 
-DigitalContact *
+DMRContact *
 TyTCodeplug::ContactElement::toContactObj() const {
-  return new DigitalContact(callType(), name(), dmrId(), ringTone());
+  return new DMRContact(callType(), name(), dmrId(), ringTone());
 }
 
 bool
-TyTCodeplug::ContactElement::fromContactObj(const DigitalContact *cont) {
+TyTCodeplug::ContactElement::fromContactObj(const DMRContact *cont) {
   if (nullptr == cont)
     return false;
 
@@ -769,7 +831,8 @@ TyTCodeplug::ZoneElement::linkZone(Zone *zone, Context &ctx) const {
 
   for (int i=0; ((i<16) && memberIndex(i)); i++) {
     if (! ctx.has<Channel>(memberIndex(i))) {
-      logWarn() << "Cannot link channel with index " << memberIndex(i) << " channel not defined.";
+      logWarn() << "Cannot link channel with index " << memberIndex(i)
+                << " to zone '" << zone->name() << "': channel not defined.";
       continue;
     }
     zone->A()->add(ctx.get<Channel>(memberIndex(i)));
@@ -830,13 +893,22 @@ TyTCodeplug::GroupListElement::setMemberIndex(unsigned n, uint16_t idx) {
 
 bool
 TyTCodeplug::GroupListElement::fromGroupListObj(const RXGroupList *lst, Context &ctx) {
-  //logDebug() << "Encode group list '" << lst->name() << "' with " << lst->count() << " members.";
   setName(lst->name());
+
+  int j=0;
+  // Iterate over all 32 entries in the codeplug
   for (int i=0; i<32; i++) {
-    if (i<lst->count()) {
-      logDebug() << "Add contact " << lst->contact(i)->name() << " to list.";
-      setMemberIndex(i, ctx.index(lst->contact(i)));
+    // Skip non-private-call entries
+    while((lst->count() > j) && (DMRContact::GroupCall != lst->contact(j)->type())) {
+      logWarn() << "Contact '" << lst->contact(i)->name() << "' in group list '" << lst->name()
+                << "' is not a group call. Skip entry.";
+      j++;
+    }
+    if (lst->count() > j) {
+      setMemberIndex(i, ctx.index(lst->contact(j)));
+      j++;
     } else {
+      // clear entry
       setMemberIndex(i, 0);
     }
   }
@@ -857,14 +929,14 @@ TyTCodeplug::GroupListElement::linkGroupListObj(RXGroupList *lst, Context &ctx)
     return false;
 
   for (int i=0; (i<32) && memberIndex(i); i++) {
-    if (! ctx.has<DigitalContact>(memberIndex(i))) {
+    if (! ctx.has<DMRContact>(memberIndex(i))) {
       logWarn() << "Cannot link contact " << memberIndex(i) << " to group list '"
                 << name() << "': Invalid contact index. Ignored.";
       continue;
     }
     //logDebug() << "Add contact idx=" << memberIndex(i) << " to group list " << lst->name() << ".";
-    if (0 > lst->addContact(ctx.get<DigitalContact>(memberIndex(i)))) {
-      logWarn() << "Cannot add contact '" << ctx.get<DigitalContact>(memberIndex(i))->name()
+    if (0 > lst->addContact(ctx.get<DMRContact>(memberIndex(i)))) {
+      logWarn() << "Cannot add contact '" << ctx.get<DMRContact>(memberIndex(i))->name()
                 << "' at idx=" << memberIndex(i) << ".";
       continue;
     }
@@ -1059,7 +1131,7 @@ TyTCodeplug::ScanListElement::linkScanListObj(ScanList *lst, Context &ctx) {
   else if (0xffff == priorityChannel1Index())
     lst->setPrimaryChannel(nullptr);
   else
-    logWarn() << "Cannot deocde reference to priority channel index " << priorityChannel1Index()
+    logWarn() << "Cannot decode reference to priority channel index " << priorityChannel1Index()
                  << " in scan list '" << name() << "'.";
 
   if (0 == priorityChannel2Index())
@@ -1069,17 +1141,17 @@ TyTCodeplug::ScanListElement::linkScanListObj(ScanList *lst, Context &ctx) {
   else if (0xffff == priorityChannel2Index())
     lst->setSecondaryChannel(nullptr);
   else
-    logWarn() << "Cannot deocde reference to secondary priority channel index " << priorityChannel2Index()
+    logWarn() << "Cannot decode reference to secondary priority channel index " << priorityChannel2Index()
               << " in scan list '" << name() << "'.";
 
   if (0 == txChannelIndex())
     lst->setRevertChannel(SelectedChannel::get());
   else if (ctx.has<Channel>(txChannelIndex()))
     lst->setRevertChannel(ctx.get<Channel>(txChannelIndex()));
-  else if (0xffff == priorityChannel2Index())
-    lst->setSecondaryChannel(nullptr);
+  else if (0xffff == txChannelIndex())
+    lst->setRevertChannel(nullptr);
   else
-    logWarn() << "Cannot deocde reference to secondary priority channel index " << txChannelIndex()
+    logWarn() << "Cannot decode reference to transmit channel index " << txChannelIndex()
                 << " in scan list '" << name() << "'.";
 
   for (int i=0; ((i<31) && memberIndex(i)); i++) {
@@ -1756,13 +1828,13 @@ TyTCodeplug::GPSSystemElement::linkGPSSystemObj(GPSSystem *sys, Context &ctx) {
   if (! isValid())
     return false;
 
-  if ((! destinationContactDisabled()) && (ctx.has<DigitalContact>(destinationContactIndex())))
-    sys->setContactObj(ctx.get<DigitalContact>(destinationContactIndex()));
+  if ((! destinationContactDisabled()) && (ctx.has<DMRContact>(destinationContactIndex())))
+    sys->setContactObj(ctx.get<DMRContact>(destinationContactIndex()));
 
   if (revertChannelIsSelected())
     sys->setRevertChannel(nullptr);
-  else if (ctx.has<Channel>(revertChannelIndex()) && ctx.get<Channel>(revertChannelIndex())->is<DigitalChannel>())
-    sys->setRevertChannel(ctx.get<Channel>(revertChannelIndex())->as<DigitalChannel>());
+  else if (ctx.has<Channel>(revertChannelIndex()) && ctx.get<Channel>(revertChannelIndex())->is<DMRChannel>())
+    sys->setRevertChannel(ctx.get<Channel>(revertChannelIndex())->as<DMRChannel>());
 
   return true;
 }
@@ -2537,13 +2609,13 @@ TyTCodeplug::EmergencySystemElement::revertChannelSelected() {
  * Implementation of TyTCodeplug::EncryptionElement
  * ******************************************************************************************** */
 TyTCodeplug::EncryptionElement::EncryptionElement(uint8_t *ptr, size_t size)
-  : Element(ptr, size)
+  : Element(ptr, size), _numEnhancedKeys(8), _numBasicKeys(16)
 {
   // pass...
 }
 
 TyTCodeplug::EncryptionElement::EncryptionElement(uint8_t *ptr)
-  : Element(ptr, 0xb0)
+  : Element(ptr, 0xb0), _numEnhancedKeys(8), _numBasicKeys(16)
 {
   // pass...
 }
@@ -2557,8 +2629,17 @@ TyTCodeplug::EncryptionElement::clear() {
   memset(_data, 0xff, 0xb0);
 }
 
+bool
+TyTCodeplug::EncryptionElement::isEnhancedKeySet(unsigned n) const {
+  QByteArray key = enhancedKey(n);
+  for (int i=0; i<16; i++) {
+    if (key[0] != key[i])
+      return true;
+  }
+  return ('\xff' != key[0]) && ('\x00' != key[0]);
+}
 QByteArray
-TyTCodeplug::EncryptionElement::enhancedKey(unsigned n) {
+TyTCodeplug::EncryptionElement::enhancedKey(unsigned n) const {
   return QByteArray((char *)(_data+n*16), 16);
 }
 void
@@ -2568,15 +2649,85 @@ TyTCodeplug::EncryptionElement::setEnhancedKey(unsigned n, const QByteArray &key
   memcpy(_data+n*16, key.constData(), 16);
 }
 
+bool
+TyTCodeplug::EncryptionElement::isBasicKeySet(unsigned n) const {
+  QByteArray key = basicKey(n);
+  for (int i=0; i<2; i++)
+    if (key[0] != key[i])
+      return true;
+  return ('\xff'!=key[0]) && ('\x00' != key[0]);
+}
 QByteArray
-TyTCodeplug::EncryptionElement::basicKey(unsigned n) {
+TyTCodeplug::EncryptionElement::basicKey(unsigned n) const {
   return QByteArray((char *)(_data+0x90+n*2), 2);
 }
 void
 TyTCodeplug::EncryptionElement::setBasicKey(unsigned n, const QByteArray &key) {
   if (2 != key.size())
     return;
-  memcpy(_data+0x90+n*16, key.constData(), 2);
+  memcpy(_data+0x90+n*2, key.constData(), 2);
+}
+
+bool
+TyTCodeplug::EncryptionElement::fromCommercialExt(CommercialExtension *encr, Context &ctx) {
+  ctx.addTable(&DMREncryptionKey::staticMetaObject);
+  ctx.addTable(&AESEncryptionKey::staticMetaObject);
+
+  // Clear all keys
+  clear();
+
+  // Encode each key
+  unsigned basicCount=0, aesCount=0;
+  for (int i=0; i<encr->encryptionKeys()->count(); i++) {
+    if (encr->encryptionKeys()->get(i)->is<DMREncryptionKey>() && (_numBasicKeys > basicCount)) {
+      DMREncryptionKey *key = encr->encryptionKeys()->get(i)->as<DMREncryptionKey>();
+      setBasicKey(basicCount++, key->key());
+      ctx.add(key, basicCount);
+    } else if (encr->encryptionKeys()->get(i)->is<AESEncryptionKey>() && (_numEnhancedKeys > aesCount)) {
+      AESEncryptionKey *key = encr->encryptionKeys()->get(i)->as<AESEncryptionKey>();
+      setEnhancedKey(aesCount++, key->key());
+      ctx.add(key, aesCount);
+    } else {
+      logWarn() << "Cannot encode key of type '"
+                << encr->encryptionKeys()->get(i)->metaObject()->className() << "'.";
+    }
+  }
+
+  return true;
+}
+
+bool
+TyTCodeplug::EncryptionElement::updateCommercialExt(Context &ctx) {
+  ctx.addTable(&DMREncryptionKey::staticMetaObject);
+  ctx.addTable(&AESEncryptionKey::staticMetaObject);
+
+  CommercialExtension *ext = ctx.config()->commercialExtension();
+  for (unsigned i=0; i<_numEnhancedKeys; i++) {
+    if (! isEnhancedKeySet(i))
+      continue;
+    AESEncryptionKey *key = new AESEncryptionKey();
+    key->setName(QString("Enhanced Key %1").arg(i+1));
+    ctx.add(key,i+1);
+    key->fromHex(enhancedKey(i).toHex());
+    ext->encryptionKeys()->add(key);
+  }
+  for (unsigned i=0; i<_numBasicKeys; i++) {
+    if (! isBasicKeySet(i))
+      continue;
+    DMREncryptionKey *key = new DMREncryptionKey();
+    key->setName(QString("Basic Key %1").arg(i+1));
+    ctx.add(key,i+1);
+    key->fromHex(basicKey(i).toHex());
+    ext->encryptionKeys()->add(key);
+  }
+
+  return ext;
+}
+
+bool
+TyTCodeplug::EncryptionElement::linkCommercialExt(CommercialExtension *ext, Context &ctx) {
+  Q_UNUSED(ext); Q_UNUSED(ctx);
+  return true;
 }
 
 
@@ -2605,8 +2756,8 @@ TyTCodeplug::index(Config *config, Context &ctx, const ErrorStack &err) const {
 
   // Map digital and DTMF contacts
   for (int i=0, d=0, a=0; i<config->contacts()->count(); i++) {
-    if (config->contacts()->contact(i)->is<DigitalContact>()) {
-      ctx.add(config->contacts()->contact(i)->as<DigitalContact>(), d+1); d++;
+    if (config->contacts()->contact(i)->is<DMRContact>()) {
+      ctx.add(config->contacts()->contact(i)->as<DMRContact>(), d+1); d++;
     } else if (config->contacts()->contact(i)->is<DTMFContact>()) {
       ctx.add(config->contacts()->contact(i)->as<DTMFContact>(), a+1); a++;
     }
@@ -2638,8 +2789,8 @@ TyTCodeplug::index(Config *config, Context &ctx, const ErrorStack &err) const {
   }
 
   // Map roaming
-  for (int i=0; i<config->roaming()->count(); i++)
-    ctx.add(config->roaming()->zone(i), i+1);
+  for (int i=0; i<config->roamingZones()->count(); i++)
+    ctx.add(config->roamingZones()->zone(i), i+1);
 
   return true;
 }
@@ -2728,6 +2879,12 @@ TyTCodeplug::encodeElements(const Flags &flags, Context &ctx, const ErrorStack &
     return false;
   }
 
+  // Define encryption keys
+  if (! this->encodePrivacyKeys(ctx.config(), flags, ctx)) {
+    errMsg(err) << "Cannot encode encryption keys.";
+    return false;
+  }
+
   // Define Channels
   if (! this->encodeChannels(ctx.config(), flags, ctx)) {
     errMsg(err) << "Cannot encode channels.";
@@ -2811,6 +2968,12 @@ TyTCodeplug::decodeElements(Context &ctx, const ErrorStack &err) {
     return false;
   }
 
+  // Decode encryption settings
+  if (! this->decodePrivacyKeys(ctx.config(), ctx, err)) {
+    errMsg(err) << "Cannot decode encryption settings.";
+    return false;
+  }
+
   // Link RX GroupLists
   if (! this->linkGroupLists(ctx, err)) {
     errMsg(err) << "Cannot link group lists.";
