@@ -34,6 +34,7 @@
 #include "zonelistview.hh"
 #include "scanlistsview.hh"
 #include "positioningsystemlistview.hh"
+#include "roamingchannellistview.hh"
 #include "roamingzonelistview.hh"
 #include "errormessageview.hh"
 #include "extensionview.hh"
@@ -274,9 +275,13 @@ Application::createMainWindow() {
   _posSysList = new PositioningSystemListView(_config);
   tabs->addTab(_posSysList, tr("GPS/APRS"));
 
+  // Wire-up "Roaming Zone List" view
+  _roamingChannelList = new RoamingChannelListView(_config);
+  tabs->addTab(_roamingChannelList, tr("Roaming Channels"));
+
   // Wire-up "Roaming Zone List" view
   _roamingZoneList = new RoamingZoneListView(_config);
-  tabs->addTab(_roamingZoneList, tr("Roaming"));
+  tabs->addTab(_roamingZoneList, tr("Roaming Zones"));
 
   // Wire-up "extension view"
   _extensionView = new ExtensionView();
@@ -481,6 +486,9 @@ Application::detectRadio() {
   if (Radio *radio = autoDetect()) {
     QMessageBox::information(nullptr, tr("Radio found"), tr("Found device '%1'.").arg(radio->name()));
     radio->deleteLater();
+  } else {
+    QMessageBox::information(nullptr, tr("No radio found"),
+                             tr("No matching device was found."));
   }
 }
 
@@ -492,9 +500,14 @@ Application::verifyCodeplug(Radio *radio, bool showSuccess) {
   // If no radio is given -> try to detect the radio
   if (nullptr == myRadio)
     myRadio = autoDetect();
-  if (nullptr == myRadio)
-    return false;
 
+  if (nullptr == myRadio) {
+    if (showSuccess) {
+      QMessageBox::warning(nullptr, tr("No radio found"),
+                           tr("No matching device was found."));
+    }
+    return false;
+  }
   Settings settings;
   RadioLimitContext ctx(settings.ignoreFrequencyLimits());
   myRadio->limits().verifyConfig(_config, ctx);
@@ -532,8 +545,11 @@ Application::downloadCodeplug() {
   }
 
   Radio *radio = autoDetect();
-  if (nullptr == radio)
+  if (nullptr == radio) {
+    QMessageBox::warning(nullptr, tr("No radio found"),
+                         tr("No matching device was found."));
     return;
+  }
 
   QProgressBar *progress = _mainWindow->findChild<QProgressBar *>("progress");
   progress->setValue(0); progress->setMaximum(100); progress->setVisible(true);
@@ -589,8 +605,11 @@ Application::uploadCodeplug() {
   Settings settings;
 
   Radio *radio = autoDetect();
-  if (nullptr == radio)
+  if (nullptr == radio) {
+    QMessageBox::warning(nullptr, tr("No radio found"),
+                         tr("No matching device was found."));
     return;
+  }
 
   if (! verifyCodeplug(radio, false)) {
     radio->deleteLater();
@@ -620,8 +639,11 @@ void
 Application::uploadCallsignDB() {
   // Start upload
   Radio *radio = autoDetect();
-  if (nullptr == radio)
+  if (nullptr == radio) {
+    QMessageBox::warning(nullptr, tr("No radio found"),
+                         tr("No matching device was found."));
     return;
+  }
 
   if (! radio->limits().hasCallSignDB()) {
     logDebug() << "Radio " << radio->name() << " does not support call-sign DB.";
