@@ -196,6 +196,15 @@ Settings::setShowExtensions(bool show) {
   setValue("showExtensions", show);
 }
 
+bool
+Settings::hideChannelNote() const {
+  return value("hideChannelNote", false).toBool();
+}
+void
+Settings::setHideChannelNote(bool hide) {
+  setValue("hideChannelNote", hide);
+}
+
 bool
 Settings::hideGSPNote() const {
   return value("hideGPSNote", false).toBool();
@@ -206,6 +215,16 @@ Settings::setHideGPSNote(bool hide) {
   setValue("hideGPSNote", hide);
 }
 
+bool
+Settings::hideRoamingChannelNote() const {
+  return value("hideRoamingChannelNote", false).toBool();
+}
+
+void
+Settings::setHideRoamingChannelNote(bool hide) {
+  setValue("hideRoamingChannelNote", hide);
+}
+
 bool
 Settings::hideRoamingNote() const {
   return value("hideRoamingNote", false).toBool();
